local visible
local mask
local shader
local tile_size = 16

function love.load()
	visible = love.graphics.newImage("visible.png")
	mask    = love.graphics.newImage("mask.png")

	-- for this algorythm it is important that textures are the same size
	-- and are divisible by tile size, so make sure of that
	assert(visible:getWidth()  == mask:getWidth())
	assert(visible:getHeight() == mask:getHeight())
	assert(visible:getWidth()  % tile_size == 0)
	assert(visible:getHeight() % tile_size == 0)

	local code = love.filesystem.read("shader.frag")
	shader = love.graphics.newShader(code)
	shader:send("debug_pos", { visible:getWidth() / 2, visible:getHeight() / 2 }) -- just initial debug ray position

	love.window.setMode(visible:getWidth(), visible:getHeight(), { resizable = true, vsync = true, borderless = false })
	love.graphics.setDefaultFilter("nearest", "nearest", 1)
end

function love.update(dt)
	local x, y = love.mouse.getPosition()

	shader:send("light_radius", 100)
	shader:send("ambient", 0.33) -- ambient lighting for areas where there is no direct light
	shader:send("light_pos", { x, y })
	shader:send("size", { visible:getWidth(), visible:getHeight() })
	shader:send("collision_mask", mask)
	shader:send("tile_size", tile_size)

	if love.mouse.isDown(1) then -- hold and move mouse 1 to visualize debug ray
		shader:send("show_debug_ray", 1)
	else
		shader:send("show_debug_ray", 0)
		shader:send("debug_pos", { x, y })
	end
end

function love.draw()
	love.graphics.setShader(shader)
	love.graphics.draw(visible)
end
